import pandas as pd
import numpy as np
from sklearn.decomposition import PCA
from Promethee import promethee
from typing import Union


def run_promethee_analysis(
    data: pd.DataFrame,
    weights: pd.DataFrame,
    criteria_min_max: pd.DataFrame,
    pref_type: Union[int, pd.DataFrame],
    q_param: Union[int, pd.DataFrame],
    p_param: Union[int, pd.DataFrame],
) -> pd.DataFrame:
    """
    Run Promethee analysis on a given dataset.
    :param data: DataFrame containing the criteria data.
    :param weights: Weights DataFrame for the criteria.
    :param criteria_min_max: Boolean DataFrame indicating if each criterion is
     to be maximized (True) or minimized (False).
    :param pref_type:
    :param q_param:
    :param p_param:
    :return: Promethee results concatenated with the original data as a
     dataframe.
    """
    # Preconditions checks
    if data.shape[1] != weights.shape[0]:
        raise ValueError(
            "Le nombre de critères dans les données doit correspondre au"
            " nombre de poids."
        )
    if data.shape[1] != criteria_min_max.shape[0]:
        raise ValueError(
            "Le nombre de critères dans les données doit correspondre au"
            " nombre de critères dans criteria_min_max."
        )
    if any(data.columns != weights.index):
        raise ValueError("Columns of data must match the index of weights.")
    # Promethee parameters
    criteria_min_max = criteria_min_max.values.flatten().tolist()
    pref_type = (
        int(pref_type.values[0][0])
        if isinstance(pref_type, pd.DataFrame)
        else int(pref_type)
    )
    q_param = (
        int(q_param.values[0][0])
        if isinstance(q_param, pd.DataFrame)
        else int(q_param)
    )
    p_param = (
        int(p_param.values[0][0])
        if isinstance(p_param, pd.DataFrame)
        else int(p_param)
    )

    # PCA
    pca = PCA()
    principal_components = pca.fit_transform(data)
    contrib_ind_df = pd.DataFrame(
        np.square(principal_components), index=data.index
    )
    loadings = pca.components_.T
    contrib_var_df = pd.DataFrame(np.square(loadings), index=data.columns)
    contrib_var_df = contrib_var_df.div(contrib_var_df.sum(axis=0), axis=1)

    # Variables weighing
    weighted_contrib = (contrib_var_df * weights.values).sum(axis=0)
    weighted_contrib = pd.DataFrame(
        np.tile(weighted_contrib.values, (contrib_ind_df.shape[0], 1)),
        index=contrib_ind_df.index,
        columns=contrib_ind_df.columns,
    )

    # Promethee call
    promethee_result_df = promethee(
        contrib_ind_df,
        weighted_contrib,
        criteria_min_max,
        pref_type,
        q_param,
        p_param,
    )
    promethee_result_df.index = data.index
    promethee_result_df.name = "PrometheeScore"

    # Final result
    res = pd.concat([promethee_result_df, data.reset_index(drop=True)], axis=1)
    return res



import matplotlib.pyplot as plt
import numpy as np
import typing
import os
from matplotlib.backends.backend_pdf import PdfPages


def generate_radar_plots(
    data, indexes: typing.Union[str, list] = None, pdf_path: str = "plots.pdf"
):
    """
    Generate radar plots for individuals selected by indices. And save them as
     PNG files.
    :param data: DataFrame containing the criteria data.
    :param indexes: List of index values corresponding to the rows to display
     (if None, all rows are displayed).
    :param pdf_path: Path of the pdf file that will be generated. Default is
     'plots'
    :return: None. Generate a pdf file with radar plots.
    """
    if not str.endswith(pdf_path, ".pdf"):
        raise ValueError("Arg 'pdf_path' must end with '.pdf'")
    path = pdf_path.split("/")
    filename = path.pop(-1)
    path = "/".join(path)
    os.makedirs(path, exist_ok=True)
    normalized_data = (data - data.min()) / (data.max() - data.min())
    with PdfPages(f"{path}/{filename}") as pdf:
        for i in indexes:
            values = normalized_data.loc[i].tolist()
            values += values[:1]  # pour fermer le polygone
            num_vars = len(data.columns)
            angles = np.linspace(
                0, 2 * np.pi, num_vars, endpoint=False
            ).tolist()
            angles += angles[:1]
            fig, ax = plt.subplots(figsize=(6, 6), subplot_kw=dict(polar=True))
            ax.plot(angles, values, linewidth=2, linestyle="solid")
            ax.fill(angles, values, alpha=0.3)
            ax.set_xticks(angles[:-1])
            ax.set_title(f"Radar plot for {i}", size=14, pad=20)
            pdf.savefig(fig)
            plt.close(fig)



import pandas as pd
from omerbio import run_promethee_analysis, generate_radar_plots
import unittest
import os
import pymupdf
import hashlib


class TestPrometheeAnalysis(unittest.TestCase):
    def test_run_promethee_analysis(self):
        data = pd.DataFrame(
            {
                "Crit1": [1, 2, 3],
                "Crit2": [4, 5, 6],
                "Crit3": [7, 8, 9],
            }
        )
        weights = pd.DataFrame(
            [0.4, 0.3, 0.3], index=["Crit1", "Crit2", "Crit3"]
        )
        criteria = pd.DataFrame([True, False, True])

        result = run_promethee_analysis(
            data, weights, criteria, pref_type=1, q_param=0, p_param=1
        )
        self.assertIn("PrometheeScore", result.columns)
        self.assertEqual(
            result["Crit1"].values.tolist(), data["Crit1"].values.tolist()
        )
        self.assertEqual(
            result["Crit2"].values.tolist(), data["Crit2"].values.tolist()
        )
        self.assertEqual(
            result["Crit3"].values.tolist(), data["Crit3"].values.tolist()
        )
        self.assertEqual(
            result["PrometheeScore"].values.round(6).tolist(),
            [0.0, 0.67, -0.67],
        )

    def test_radar_plot(self):
        data = pd.DataFrame(
            {
                "Crit1": [1, 2, 3],
                "Crit2": [4, 5, 6],
                "Crit3": [7, 8, 9],
                "Crit4": [10, 2, 5],
                "Crit5": [7, 9, 6],
                "Crit6": [7, 8, 23],
            },
            index=["A", "B", "C"],
        )
        generate_radar_plots(
            data,
            indexes=["A", "B", "C"],
            pdf_path="tests/test_tmp/test_plots.pdf",
        )
        self.assertTrue(os.path.exists("tests/test_tmp/test_plots.pdf"))
        output_file = pymupdf.open("tests/test_tmp/test_plots.pdf")
        expected_hash = [
            "995a6ca83a5ae01c15450e21fd030d78",
            "1f4cd47a9472df7ca784e167fe804b5d",
            "0e043ddf1888ff02331c2f83d4660482",
        ]
        self.assertEqual(output_file.page_count, 3)
        for i in range(3):
            self.assertEqual(
                hashlib.md5(
                    output_file.load_page(i).get_pixmap(dpi=24).tobytes()
                ).hexdigest(),
                expected_hash[i],
            )
        os.remove("tests/test_tmp/test_plots.pdf")
        os.rmdir("tests/test_tmp")


if __name__ == "__main__":
    unittest.main()
